package ru.tularegion.corp.ui.components

import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.material3.DropdownMenuItem
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.ExposedDropdownMenuBox
import androidx.compose.material3.MenuAnchorType
import androidx.compose.material3.OutlinedTextField
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.MutableState
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.ui.Modifier
import androidx.compose.ui.input.pointer.PointerIcon
import androidx.compose.ui.input.pointer.pointerHoverIcon
import androidx.compose.ui.text.style.TextOverflow
import kotlin.enums.EnumEntries

interface DropdownSelect {
    val id: Int
    val title: String
}

enum class RedirectMode(
    override val id: Int,
    override val title: String,
) : DropdownSelect {
    NO_REDIRECT(0, "Отключить"),
    RU_TO_LOCAL(1, "Перенаправлять RU на LOCAL"),
    LOCAL_TO_RU(2, "Перенаправлять LOCAL на RU"),
}

enum class CopyMode(
    override val id: Int,
    override val title: String,
) : DropdownSelect {
    NO_REPLACE(0, "Отключить"),
//    RU_TO_LOCAL(1, "Заменять RU на LOCAL"),
//    LOCAL_TO_RU(2, "Заменять LOCAL на RU"),
}

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun <T>DropdownSelect(
    title: String,
    caption: String,
    items: EnumEntries<T>,
    valueState: MutableState<T>,
    onValueChange: (Int) -> Unit,
) where T : Enum<T>, T : DropdownSelect {
    val dropdownMenuState = remember {
        mutableStateOf(false)
    }
    ExposedDropdownMenuBox(
        expanded = dropdownMenuState.value,
        onExpandedChange = {
            dropdownMenuState.value = it
        },
    ) {
        OutlinedTextField(
            value = valueState.value.title,
            onValueChange = {},
            modifier = Modifier
                .fillMaxWidth()
                .menuAnchor(
                    type = MenuAnchorType.PrimaryEditable,
                )
                .pointerHoverIcon(
                    icon = PointerIcon.Default,
                    overrideDescendants = true,
                ),
            label = {
                Text(
                    text = title,
                    overflow = TextOverflow.Ellipsis,
                    softWrap = false,
                )
            },
            supportingText = {
                Text(
                    text = caption,
                )
            },
            readOnly = true,
            singleLine = true,
        )
        ExposedDropdownMenu(
            expanded = dropdownMenuState.value,
            onDismissRequest = {
                dropdownMenuState.value = false
            }
        ) {
            items.forEach {
                DropdownMenuItem(
                    text = {
                        Text(
                            text = it.title,
                        )
                    },
                    onClick = {
                        onValueChange(it.id)
                        valueState.value = it
                        dropdownMenuState.value = false
                    },
                )
            }
        }
    }
}