package ru.tularegion.corp.ui.components

import androidx.compose.animation.AnimatedVisibility
import androidx.compose.animation.animateColorAsState
import androidx.compose.animation.core.animateDpAsState
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.ColumnScope
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material3.Card
import androidx.compose.material3.CardColors
import androidx.compose.material3.CardDefaults
import androidx.compose.material3.HorizontalDivider
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Switch
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.MutableState
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.text.style.TextOverflow
import androidx.compose.ui.unit.dp

@Composable
fun SwitchSpoiler(
    title: String,
    caption: String,
    enabledState: MutableState<Boolean>,
    onValueChange: (Boolean) -> Unit,
    content: @Composable ColumnScope.() -> Unit,
) {
    val backgroundColor = animateColorAsState(
        targetValue = when (enabledState.value) {
            true -> MaterialTheme.colorScheme.primary.copy(
                alpha = 0.12f,
            )
            false -> Color.Transparent
        },
    )
    val horizontalPadding = animateDpAsState(
        targetValue = when (enabledState.value) {
            true -> 16.dp
            false -> 0.dp
        },
    )
    val shapeState = animateDpAsState(
        targetValue = when (enabledState.value) {
            true -> 28.0.dp
            false -> 0.0.dp
        },
    )
    val cardColors = CardDefaults.cardColors()
    Card(
        modifier = Modifier
            .padding(
                horizontal = horizontalPadding.value,
            )
            .fillMaxWidth(),
        shape = RoundedCornerShape(shapeState.value),
        colors = CardColors(
            containerColor = backgroundColor.value,
            contentColor = cardColors.contentColor,
            disabledContainerColor = cardColors.disabledContainerColor,
            disabledContentColor = cardColors.disabledContentColor,
        ),
    ) {
        Column {
            Row(
                modifier = Modifier
                    .fillMaxWidth()
                    .clickable(
                        onClick = {
                            val value = !enabledState.value
                            onValueChange(value)
                            enabledState.value = value
                        },
                    )
                    .padding(16.dp),
                horizontalArrangement = Arrangement.SpaceBetween,
                verticalAlignment = Alignment.CenterVertically,
            ) {
                Column(
                    verticalArrangement = Arrangement.spacedBy(4.dp),
                ) {
                    Text(
                        text = title,
                        overflow = TextOverflow.Ellipsis,
                        softWrap = false,
                        style = MaterialTheme.typography.titleLarge,
                    )
                    Text(
                        text = caption,
                        overflow = TextOverflow.Ellipsis,
                        style = MaterialTheme.typography.bodyMedium,
                    )
                }
                Switch(
                    checked = enabledState.value,
                    onCheckedChange = {
                        onValueChange(it)
                        enabledState.value = it
                    },
                )
            }

            AnimatedVisibility(
                visible = enabledState.value,
            ) {
                Column {
                    HorizontalDivider()
                    Column(
                        content = content,
                    )
                }
            }
        }
    }
}