package ru.tularegion.corp.ui.screens

import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.material3.OutlinedTextField
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.ui.Modifier
import androidx.compose.ui.text.input.PasswordVisualTransformation
import androidx.compose.ui.unit.dp
import ru.tularegion.corp.backend.JsCallbacks
import ru.tularegion.corp.backend.setAutoAuthLogin
import ru.tularegion.corp.backend.setAutoAuthMode
import ru.tularegion.corp.backend.setAutoAuthPassword
import ru.tularegion.corp.backend.setCopyMode
import ru.tularegion.corp.backend.setRedirectMode
import ru.tularegion.corp.ui.components.CopyMode
import ru.tularegion.corp.ui.components.DropdownSelect
import ru.tularegion.corp.ui.components.RedirectMode
import ru.tularegion.corp.ui.components.SwitchSpoiler

@Composable
fun SettingsScreen() {
    val redirectModeState = remember {
        mutableStateOf(RedirectMode.NO_REDIRECT)
    }
    JsCallbacks.redirectMode = { value ->
        redirectModeState.value = RedirectMode.entries.find {
            it.id == value
        } ?: RedirectMode.NO_REDIRECT
    }

    val copyModeState = remember {
        mutableStateOf(CopyMode.NO_REPLACE)
    }
    JsCallbacks.copyMode = { value ->
        copyModeState.value = CopyMode.entries.find {
            it.id == value
        } ?: CopyMode.NO_REPLACE
    }

    val autoAuthModeState = remember {
        mutableStateOf(false)
    }
    JsCallbacks.autoAuthMode = {
        autoAuthModeState.value = it
    }

    val loginState = remember {
        mutableStateOf("")
    }
    JsCallbacks.autoAuthLogin = {
        loginState.value = it
    }

    val passwordState = remember {
        mutableStateOf("")
    }
    JsCallbacks.autoAuthPassword = {
        passwordState.value = it
    }

    Column {
        Column(
            modifier = Modifier
                .padding(
                    horizontal = 16.dp,
                ),
            verticalArrangement = Arrangement.spacedBy(16.dp),
        ) {
            DropdownSelect(
                title = "Перенаправление между RU и LOCAL",
                caption = "Перенаправлять все страницы на RU, либо на LOCAL",
                items = RedirectMode.entries,
                valueState = redirectModeState,
                onValueChange = {
                    setRedirectMode(it)
                },
            )
            DropdownSelect(
                title = "Замена ссылок при копировании",
                caption = "Заменять все ссылки при копировании на RU, либо на LOCAL",
                items = CopyMode.entries,
                valueState = copyModeState,
                onValueChange = {
                    setCopyMode(it)
                },
            )
        }

        SwitchSpoiler(
            title = "Автоматический вход",
            caption = "Выполнять вход на портал автоматически",
            enabledState = autoAuthModeState,
            onValueChange = {
                setAutoAuthMode(it)
            },
        ) {
            Column(
                modifier = Modifier
                    .padding(16.dp),
                verticalArrangement = Arrangement.spacedBy(16.dp),
            ) {
                OutlinedTextField(
                    value = loginState.value,
                    onValueChange = {
                        setAutoAuthLogin(it)
                        loginState.value = it
                    },
                    modifier = Modifier
                        .fillMaxWidth(),
                    label = {
                        Text(
                            text = "Логин",
                        )
                    },
                    singleLine = true,
                )
                OutlinedTextField(
                    value = passwordState.value,
                    onValueChange = {
                        setAutoAuthPassword(it)
                        passwordState.value = it
                    },
                    modifier = Modifier
                        .fillMaxWidth(),
                    label = {
                        Text(
                            text = "Пароль",
                        )
                    },
                    visualTransformation = PasswordVisualTransformation(),
                    singleLine = true,
                )
            }
        }
    }
}